const BASE_URL="https://service.nosebrain.de/service/uni-wue/";

const EMOJI_LOOKUP={
  "Alkohol": "🍸",
  "Fisch": "🐟",
  "Fleischlos": "🌽",
  "Geflügel": "🐔",
  "Kalb": "🐄",
  "Lamm": "🐏",
  "Rind": "🐄",
  "Schwein": "🐖",
  "Vegan": "🌱",
  "Vorderschinken": "🍖",
  "Wild": "🦌",
  "Burger": "🍔"
};

const WEEKDAYS = [ "Montag", "Dienstag", "Mittwoch", "Donnerstag", "Freitag" ];

const LOCATIONS = [
  //{name: "Frankenstube", id: 1},
  //{name: "Mensa", id: 2},
  {name: "Mensateria", id: 6},
  {name: "Outer-Rim-Mensa", id: 7},
  {name: "Essensausgabe Philo", id: 8}
];

const ENABLE_ALARMS = false

const build_plan_from_json = function(json, weekday) {
  let f;
  const data = JSON.parse(json);
  if(data.message) {
    return data.message;
  }
  if (!data.days) {
    console.log("Error reading data");
    console.log(data);
    return "Error reading json from server. Please try again later";
  }
  const day = data.days[weekday];
  if (day.holiday) {
    return "(geschlossen)";
  } else {
    let foods = [];
    for (f of day.food) {
      foods.push(`${to_emojis(f)} ${f.description}: ${f.prices[0]} / ${f.prices[1]} / ${f.prices[2]}`);
    }

    if (ENABLE_ALARMS) {
      if (day.food.some(f => /burger/i.test(f.description))) {
        foods = ["🍔🍔🍔🍔🍔🍔 *BURGERALARM*  🍔🍔🍔🍔🍔🍔\n"].concat(foods);
      }

      if (day.food.some(f => /currywurst/i.test(f.description))) {
        foods = ["🔔🔔🔔🔔🔔🔔 *CURRYWURST* 🔔🔔🔔🔔🔔🔔\n"].concat(foods);
      }
    }

    return foods.join("\n");
  }
};


var to_emojis = food => (food.ingredients.map((i) => EMOJI_LOOKUP[i])).join(" ");
    
const day_name = function(weekday, offset) {
  if (offset === 0) {
    return "heute";
  } else if (offset === 1) {
    return "morgen";
  } else if (offset === -1) {
    return "gestern";
  } else if (offset < 0) {
    return `letzten ${WEEKDAYS[weekday]}`;
  } else {
    return WEEKDAYS[weekday];
  }
};

const retrievePlan = (robot, location, weekday) =>
  new Promise(function(resolve) {
    return robot.http(BASE_URL + location.id + "/CURRENT.json")
      .header('Accept', 'application/json')
      .get()((err, res, body) => resolve(`*${location.name}*:\n${build_plan_from_json(body, weekday)}`));
  })
;

const mensaplan = function(robot, response) {
  let offset = response.match[1] * 1;
  if (isNaN(offset)) { offset = 0; }
  const weekday = (new Date().getDay() - 1) + offset;
  if ((weekday < 0) || (weekday > 4)) {
    response.send("Ungültiger Offset! Nur aktuelle Woche Montag - Freitag.");
    return;
  }

  const day = day_name(weekday, offset);

  return Promise.all(LOCATIONS.map(loc => retrievePlan(robot, loc, weekday)))
    .then(plans => response.send(`Speiseplan für ${day}:\n${plans.join("\n")}`));
};

// here we call the hubot api to react to messages
module.exports = function(robot) {
  // someone mentioning the bot and saying !mensa
  robot.respond(/!mensa/i, response => mensaplan(robot, response));
  // any message starting with !mensa, optional offset
  return robot.hear(/^!mensa ?(\S*)/i, response => mensaplan(robot, response));
};
